@@ -10,6 +10,11 @@
 
 #define HEX_RESTORE_UTF8	2
 #define HEX_RESTORE_CRLF	4
+#define HEX_RESTORE_INSERT	8
+#define HEX_RESTORE_WORDWRAP	16
+#define HEX_RESTORE_AUTOINDENT	32
+#define HEX_RESTORE_ANSI	64
+#define HEX_RESTORE_PICTURE	128
 #define OPT_BUF_SIZE 300
 
 const char *aborthint = "^C";
@@ -69,6 +74,7 @@ OPTIONS pdefault = {
 	0,		/* tex_comment */
 	0,		/* hex */
 	0,		/* hide ansi */
+	0,		/* status line context */
 	NULL,		/* text_delimiters */
 	NULL,		/* Characters which can indent paragraphs */
 	NULL,		/* Characters which begin non-paragraph lines */
@@ -131,6 +137,7 @@ OPTIONS fdefault = {
 	0,		/* tex_comment */
 	0,		/* hex */
 	0,		/* hide ansi */
+	0,		/* status line context */
 	NULL,		/* text_delimiters */
 	">;!#%/",	/* Characters which can indent paragraphs */
 	".",	/* Characters which begin non-paragraph lines */
@@ -199,6 +206,31 @@ void lazy_opts(B *b, OPTIONS *o)
 			b->o.crlf = 0;
 			b->o.hex |= HEX_RESTORE_CRLF;
 		}
+
+		if (!b->o.overtype) {
+			b->o.overtype = 1;
+			b->o.hex |= HEX_RESTORE_INSERT;
+		}
+
+		if (b->o.wordwrap) {
+			b->o.wordwrap = 0;
+			b->o.hex |= HEX_RESTORE_WORDWRAP;
+		}
+
+		if (b->o.autoindent) {
+			b->o.autoindent = 0;
+			b->o.hex |= HEX_RESTORE_AUTOINDENT;
+		}
+
+		if (b->o.ansi) {
+			b->o.ansi = 0;
+			b->o.hex |= HEX_RESTORE_ANSI;
+		}
+
+		if (b->o.picture) {
+			b->o.picture = 0;
+			b->o.hex |= HEX_RESTORE_PICTURE;
+		}
 	}
 	
 }
@@ -268,6 +300,7 @@ struct glopts {
 	{"overwrite",4, NULL, (char *) &fdefault.overtype, _("Overtype mode"), _("Insert mode"), _("Overtype mode"), 0, 0, 0 },
 	{"hex",4, NULL, (char *) &fdefault.hex, _("Hex edit mode"), _("Text edit mode"), _("Hex edit display mode"), 0, 0, 0 },
 	{"ansi",4, NULL, (char *) &fdefault.ansi, _("Hide ANSI sequences"), _("Reveal ANSI sequences"), _("Hide ANSI mode"), 0, 0, 0 },
+	{"title",4, NULL, (char *) &fdefault.title, _("Status line context enabled"), _("Status line context disabled"), _("Status line context display mode"), 0, 0, 0 },
 	{"autoindent",	4, NULL, (char *) &fdefault.autoindent, _("Autoindent enabled"), _("Autoindent disabled"), _("Autoindent mode"), 0, 0, 0 },
 	{"wordwrap",	4, NULL, (char *) &fdefault.wordwrap, _("Wordwrap enabled"), _("Wordwrap disabled"), _("Word wrap mode"), 0, 0, 0 },
 	{"tab",	14, NULL, (char *) &fdefault.tab, _("Tab width (%lld): "), 0, _("Tab width"), 0, 1, 64 },
@@ -1034,7 +1067,7 @@ static int doencoding(W *w, char *s, void *obj, int *notify)
 	return ret;
 }
 
-char **encodings = NULL; /* Array of available encodinges */
+char **encodings = NULL; /* Array of available encodings */
 
 static int encodingcmplt(BW *bw, int k)
 {
@@ -1108,6 +1141,31 @@ static int olddoopt(BW *bw, int y, int flg, int *notify)
 						bw->o.crlf = 0;
 						bw->o.hex |= HEX_RESTORE_CRLF;
 					}
+
+					if (!bw->o.overtype) {
+						bw->o.overtype = 1;
+						bw->o.hex |= HEX_RESTORE_INSERT;
+					}
+
+					if (bw->o.wordwrap) {
+						bw->o.wordwrap = 0;
+						bw->o.hex |= HEX_RESTORE_WORDWRAP;
+					}
+
+					if (bw->o.autoindent) {
+						bw->o.autoindent = 0;
+						bw->o.hex |= HEX_RESTORE_AUTOINDENT;
+					}
+
+					if (bw->o.ansi) {
+						bw->o.ansi = 0;
+						bw->o.hex |= HEX_RESTORE_ANSI;
+					}
+
+					if (bw->o.picture) {
+						bw->o.picture = 0;
+						bw->o.hex |= HEX_RESTORE_PICTURE;
+					}
 					/* Try to put entire hex dump on screen in case where we were
 					   scrolled far to the right */
 					bw->offset = 0;
@@ -1121,6 +1179,22 @@ static int olddoopt(BW *bw, int y, int flg, int *notify)
 						/* Turn CRLF back on */
 						bw->o.crlf = 1;
 					}
+
+					if (oldval & HEX_RESTORE_INSERT) {
+						bw->o.overtype = 0;
+					}
+					if (oldval & HEX_RESTORE_WORDWRAP) {
+						bw->o.wordwrap = 1;
+					}
+					if (oldval & HEX_RESTORE_AUTOINDENT) {
+						bw->o.autoindent = 1;
+					}
+					if (oldval & HEX_RESTORE_ANSI) {
+						bw->o.ansi = 1;
+					}
+					if (oldval & HEX_RESTORE_PICTURE) {
+						bw->o.picture = 1;
+					}
 					/* Update column in case we moved while in hex mode */
 					bw->cursor->xcol = piscol(bw->cursor);
 				}
