@@ -574,12 +574,14 @@ int ublkcpy(W *w, int k)
 			}
 
 			binsb(bw->cursor, tmp);
-			if (lightoff)
-				unmark(bw->parent, 0);
-			else {
-				umarkb(bw->parent, 0);
-				umarkk(bw->parent, 0);
-				pfwrd(markk, size);
+			if (k != -2) { /* Mouse - leave selection where it is to more closely match X */
+				if (lightoff)
+					unmark(bw->parent, 0);
+				else {
+					umarkb(bw->parent, 0);
+					umarkk(bw->parent, 0);
+					pfwrd(markk, size);
+				}
 			}
 			updall();
 			return 0;
@@ -1117,12 +1119,12 @@ int ufilt(W *w, int k)
 #else
 	switch (checkmark(bw)) {
 	case 0:
-		if (wmkpw(bw->parent, joe_gettext(_("Command to filter block through (^C to abort): ")), &filthist, dofilt, NULL, NULL, utypebw, NULL, NULL, locale_map, 1))
+		if (wmkpw(bw->parent, joe_gettext(_("Command to filter block through (%{abort} to abort): ")), &filthist, dofilt, NULL, NULL, cmplt_command, NULL, NULL, locale_map, 1))
 			return 0;
 		else
 			return -1;
 	case 1:
-		if (wmkpw(bw->parent, joe_gettext(_("Command to filter file through (^C to abort): ")), &filthist, dofilt, NULL, NULL, utypebw, NULL, NULL, locale_map, 1))
+		if (wmkpw(bw->parent, joe_gettext(_("Command to filter file through (%{abort} to abort): ")), &filthist, dofilt, NULL, NULL, cmplt_command, NULL, NULL, locale_map, 1))
 			return 0;
 		else
 			return -1;
@@ -1198,21 +1200,12 @@ int uupper(W *w, int k)
 }
 
 /* Get sum, sum of squares, and return count of
- * a block of numbers.
- *
- * avg = sum/count
- *
- * stddev = sqrt(  (a-avg)^2 + (b-avg)^2 + (c-avg)^2 )
- *        = sqrt(  a^2-2*a*avg+avg^2  + b^2-2*b*avg+avg^2 +c^2-2*c*avg+avg^2 )
- *        = sqrt(  a^2+b^2+c^2 + 3*avg^2 - 2*avg*(a+b+c) )
- *        = sqrt(  sumsq + count*avg^2 - 2*avg*sum  )
- *
- */
+ * a block of numbers. */
 
-int blksum(double *sum, double *sumsq)
+int blksum(BW *bw, double *sum, double *sumsq)
 {
 	char buf[80];
-	if (markv(1)) {
+	if (checkmark(bw) != 2) {
 		P *q = pdup(markb, "blksum");
 		int x;
 		int c;
@@ -1237,8 +1230,8 @@ int blksum(double *sum, double *sumsq)
 						c=pgetc(q);
 						if ((c >= '0' && c <= '9') || c == 'e' || c == 'E' ||
 						    c == 'p' || c == 'P' || c == 'x' || c == 'X' ||
-						    c == '.' || c == '-' || c == '+' ||
-						    (c >= 'a' && c <= 'f') || (c >= 'A' && c<='F')) {
+						    c == '.' || c == '-' || c == '+' || c == 'o' || c == 'O' ||
+						    (c >= 'a' && c <= 'f') || (c >= 'A' && c<='F') || (c == '_')) {
 							if(x != 79)
 								buf[x++]= TO_CHAR_OK(c);
 						} else
@@ -1246,7 +1239,7 @@ int blksum(double *sum, double *sumsq)
 					}
 					/* Convert number to floating point, add it to total */
 					buf[x] = 0;
-					v = strtod(buf,NULL);
+					v = joe_strtod(buf,NULL);
 					++count;
 					accu += v;
 					accusq += v*v;
@@ -1257,18 +1250,99 @@ int blksum(double *sum, double *sumsq)
 		prm(q);
 		*sum = accu;
 		*sumsq = accusq;
+		if (filtflg)
+			unmark(bw->parent, 0);
 		return count;
 	} else
 		return -1;
 }
 
+int blklr(BW *bw, double *xsum, double *xsumsq, double *ysum, double *ysumsq, double *xy, int logx, int logy)
+{
+	char buf[80];
+	if (checkmark(bw) != 2) {
+		P *q = pdup(markb, "blklr");
+		int x;
+		int c;
+		double accux = 0.0;
+		double accuxsq = 0.0;
+		double accuy = 0.0;
+		double accuysq = 0.0;
+		double accuxy = 0.0;
+		double prevx = 0.0;
+		double v;
+		int state = 0; /* 0 = x, 1 = y */
+		int count = 0;
+		off_t left = markb->xcol;
+		off_t right = markk->xcol;
+		while (q->byte < markk->byte) {
+			/* Skip until we're within columns */
+			while (q->byte < markk->byte && square && (piscol(q) < left || piscol(q) >= right))
+				pgetc(q);
+
+			/* Skip to first number */
+			while (q->byte < markk->byte && (!square || (piscol(q) >= left && piscol(q) < right))) {
+				c=pgetc(q);
+				if ((c >= '0' && c <= '9') || c == '.' || c == '-') {
+					/* Copy number into buffer */
+					buf[0] = TO_CHAR_OK(c); x=1;
+					while (q->byte < markk->byte && (!square || (piscol(q) >= left && piscol(q) < right))) {
+						c=pgetc(q);
+						if ((c >= '0' && c <= '9') || c == 'e' || c == 'E' ||
+						    c == 'p' || c == 'P' || c == 'x' || c == 'X' ||
+						    c == '.' || c == '-' || c == '+' || c == 'o' || c == 'O' ||
+						    (c >= 'a' && c <= 'f') || (c >= 'A' && c<='F') || (c == '_')) {
+							if(x != 79)
+								buf[x++]= TO_CHAR_OK(c);
+						} else
+							break;
+					}
+					/* Convert number to floating point, add it to total */
+					buf[x] = 0;
+					v = joe_strtod(buf,NULL);
+					if (!state) {
+						if (logx)
+							v = log(v);
+						prevx = v;
+						accux += v;
+						accuxsq += v*v;
+						state = 1;
+					} else {
+						if (logy)
+							v = log(v);
+						accuy += v;
+						accuysq += v*v;
+						accuxy += prevx*v;
+						state = 0;
+						++count;
+					}
+					break;
+				}
+			}
+		}
+		prm(q);
+		*xsum = accux;
+		*xsumsq = accuxsq;
+		*ysum = accuy;
+		*ysumsq = accuysq;
+		*xy = accuxy;
+		if (filtflg)
+			unmark(bw->parent, 0);
+		if (state)
+			return -1;
+		else
+			return count;
+	} else
+		return -1;
+}
+
 /* Get a (possibly square) block into a buffer
  * Block is converted to UTF-8
  */
 
-char *blkget()
+char *blkget(BW *bw)
 {
-	if (markv(1)) {
+	if (checkmark(bw) != 2) {
 		P *q;
 		ptrdiff_t buf_size = markk->byte - markb->byte + 1; /* Risky... */
 		ptrdiff_t buf_x = 0;
@@ -1308,6 +1382,8 @@ char *blkget()
 		}
 		prm(q);
 		buf[buf_x] = 0;
+		if (filtflg)
+			unmark(bw->parent, 0);
 		return buf;
 	} else
 		return 0;
